# pcop.R
# Compute the Principal Curve of Oriented Points (Delicado 2001, Delicado & Huerta 2003)
# for a given p-dimensional data matrix x of lenght n
#
# Arguments:
#         x: a matrix of n points in dimension p
#       C_H: The smoothing parameter h is C_H times the value given by the normal reference rule.
#            Default value: 0.75. Constraints: 0.5 <= C_H <=  1.5 
#       C_D: The distance between two consecutive principal oriented points in a PCOP is about C_D times 
#            the value of the smoothing parameter h.
#            Default value: 0.3. Constraints: 0.25 <= C_D <= .5
# plot.true: if TRUE, the function produce a plot
#       ...: Additional parameters passed to function "get_lam"
#
# Require: princurve, application "PCOP.exe"
pcop <- function(x,Ch=1.5,Cd=.3,plot.true=FALSE,...){
   x <- round(x,8)
   write.table(x,file="proyec.dat",col.names=FALSE,row.names=FALSE,append=FALSE)
#   system("del setup.dat")

   C_H <- paste("C_H ",Ch,sep="")
   C_D <- paste("C_D ",Cd,sep="")
   cat(file="setup.dat", "datafile proyec.dat", "outfile proyec.alpha", C_H, C_D, sep="\n", append=FALSE)

   system("PCOP.exe", invisible = TRUE)

#   read.table("proyec.dat")
#   system("rm proyec.dat")

   proyec <- read.table("proyec.alpha")
   pr.curve <- proyec[,-1]

# Format 1 for the output principal curve.
# ----------------------------------------
# The structure of the data.frame "proyec" is as follows: 
# There is a line for each identified principal oriented point. 
# The columns are:
# 1. Value of the parameter t such the the principal oriented point is PCOP(t).
# 2. Density estimation for the random variable induced over the PCOP at t.
# 3. Span: proportion of original data involved in the determination of the principal oriented point.
# 4. Variance over the hyperplane orthogonal to the PCOP at the principal oriented point.
# 5:(5+p-1). The p coordinates of the principal oriented point.
# (5+p):(5+2p-1). The p coordinates of the principal direction for the principal oriented point.


p <- dim(x)[2]

names(pr.curve) <- c("param", "dens", "span", "orth.var", 
              paste("pop", 1:p, sep = ""), 
              paste("pr.dir", 1:p, sep = ""))
# pr.curve: principal curve of oriented points 
#           with original format (format 1)

# pc: principal curve of oriented points 
#     with format as in HS's "princurv" library (format 2)
s<-as.matrix(pr.curve[,5:(5+p-1)])
pc <- get.lam(x,s,...)

if (plot.true) {
    plot(x[, 1:2], xlim = adjust.range(x[, 1], 1.4), ylim = adjust.range(x[, 
        2], 1.4))
      lines(pc$s[pc$tag, 1:2])
}


return(list(pcop.f1=pr.curve,pcop.f2=pc))
}
